# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from fxsdk.x.fx.dex.v1 import query_pb2 as fx_dot_dex_dot_v1_dot_query__pb2


class QueryStub(object):
    """Query defines the gRPC querier service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.QueryParams = channel.unary_unary(
                '/fx.dex.v1.Query/QueryParams',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryParamsReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryParamsResp.FromString,
                )
        self.QueryOrders = channel.unary_unary(
                '/fx.dex.v1.Query/QueryOrders',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrdersRequest.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrdersResponse.FromString,
                )
        self.QueryOrder = channel.unary_unary(
                '/fx.dex.v1.Query/QueryOrder',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderRequest.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderResponse.FromString,
                )
        self.QueryOrderbook = channel.unary_unary(
                '/fx.dex.v1.Query/QueryOrderbook',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderbookReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderbookResp.FromString,
                )
        self.QueryPositions = channel.unary_unary(
                '/fx.dex.v1.Query/QueryPositions',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionsReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionsResp.FromString,
                )
        self.QueryPosition = channel.unary_unary(
                '/fx.dex.v1.Query/QueryPosition',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionResp.FromString,
                )
        self.QueryFunding = channel.unary_unary(
                '/fx.dex.v1.Query/QueryFunding',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingResp.FromString,
                )
        self.QueryPairFundingRates = channel.unary_unary(
                '/fx.dex.v1.Query/QueryPairFundingRates',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPairFundingRatesReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPairFundingRatesResp.FromString,
                )
        self.QueryFundingTime = channel.unary_unary(
                '/fx.dex.v1.Query/QueryFundingTime',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingTimeReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingTimeResp.FromString,
                )
        self.QueryDealPrice = channel.unary_unary(
                '/fx.dex.v1.Query/QueryDealPrice',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryDealPriceReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryDealPriceResp.FromString,
                )
        self.QueryMatchResult = channel.unary_unary(
                '/fx.dex.v1.Query/QueryMatchResult',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMatchResultReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMatchResultResp.FromString,
                )
        self.QueryMarkPrice = channel.unary_unary(
                '/fx.dex.v1.Query/QueryMarkPrice',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkPriceReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkPriceResp.FromString,
                )
        self.QueryMarkAndOraclePrice = channel.unary_unary(
                '/fx.dex.v1.Query/QueryMarkAndOraclePrice',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkAndOraclePriceReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkAndOraclePriceResp.FromString,
                )
        self.QueryLiquidationPrice = channel.unary_unary(
                '/fx.dex.v1.Query/QueryLiquidationPrice',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryLiquidationPriceReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryLiquidationPriceResp.FromString,
                )
        self.QueryMovingAverage = channel.unary_unary(
                '/fx.dex.v1.Query/QueryMovingAverage',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMAReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMAResp.FromString,
                )
        self.QueryStoreStatistic = channel.unary_unary(
                '/fx.dex.v1.Query/QueryStoreStatistic',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryStoreStatisticReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryStoreStatisticResp.FromString,
                )
        self.QueryIsFunding = channel.unary_unary(
                '/fx.dex.v1.Query/QueryIsFunding',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryIsFundingReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryIsFundingResp.FromString,
                )
        self.QueryPremiumIndexConf = channel.unary_unary(
                '/fx.dex.v1.Query/QueryPremiumIndexConf',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPremiumIndexConfReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPremiumIndexConfResp.FromString,
                )
        self.QueryNeedToLiquidationPosIds = channel.unary_unary(
                '/fx.dex.v1.Query/QueryNeedToLiquidationPosIds',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryNeedToLiquidationPosIdsReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryNeedToLiquidatorPosIdsResp.FromString,
                )
        self.QueryAccountNumber = channel.unary_unary(
                '/fx.dex.v1.Query/QueryAccountNumber',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryAccountNumberReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryAccountNumberResp.FromString,
                )
        self.QueryReserve = channel.unary_unary(
                '/fx.dex.v1.Query/QueryReserve',
                request_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryReserveReq.SerializeToString,
                response_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryReserveResp.FromString,
                )


class QueryServicer(object):
    """Query defines the gRPC querier service.
    """

    def QueryParams(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryOrders(self, request, context):
        """orders query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryOrder(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryOrderbook(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryPositions(self, request, context):
        """positions query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryPosition(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryFunding(self, request, context):
        """fundings query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryPairFundingRates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryFundingTime(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryDealPrice(self, request, context):
        """query latest deal price no matter what block
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryMatchResult(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryMarkPrice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryMarkAndOraclePrice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryLiquidationPrice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryMovingAverage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryStoreStatistic(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryIsFunding(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryPremiumIndexConf(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryNeedToLiquidationPosIds(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryAccountNumber(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryReserve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'QueryParams': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryParams,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryParamsReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryParamsResp.SerializeToString,
            ),
            'QueryOrders': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryOrders,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrdersRequest.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrdersResponse.SerializeToString,
            ),
            'QueryOrder': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryOrder,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderRequest.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderResponse.SerializeToString,
            ),
            'QueryOrderbook': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryOrderbook,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderbookReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderbookResp.SerializeToString,
            ),
            'QueryPositions': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryPositions,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionsReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionsResp.SerializeToString,
            ),
            'QueryPosition': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryPosition,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionResp.SerializeToString,
            ),
            'QueryFunding': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryFunding,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingResp.SerializeToString,
            ),
            'QueryPairFundingRates': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryPairFundingRates,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPairFundingRatesReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPairFundingRatesResp.SerializeToString,
            ),
            'QueryFundingTime': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryFundingTime,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingTimeReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingTimeResp.SerializeToString,
            ),
            'QueryDealPrice': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryDealPrice,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryDealPriceReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryDealPriceResp.SerializeToString,
            ),
            'QueryMatchResult': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryMatchResult,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMatchResultReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMatchResultResp.SerializeToString,
            ),
            'QueryMarkPrice': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryMarkPrice,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkPriceReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkPriceResp.SerializeToString,
            ),
            'QueryMarkAndOraclePrice': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryMarkAndOraclePrice,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkAndOraclePriceReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkAndOraclePriceResp.SerializeToString,
            ),
            'QueryLiquidationPrice': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryLiquidationPrice,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryLiquidationPriceReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryLiquidationPriceResp.SerializeToString,
            ),
            'QueryMovingAverage': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryMovingAverage,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMAReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryMAResp.SerializeToString,
            ),
            'QueryStoreStatistic': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryStoreStatistic,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryStoreStatisticReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryStoreStatisticResp.SerializeToString,
            ),
            'QueryIsFunding': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryIsFunding,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryIsFundingReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryIsFundingResp.SerializeToString,
            ),
            'QueryPremiumIndexConf': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryPremiumIndexConf,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPremiumIndexConfReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryPremiumIndexConfResp.SerializeToString,
            ),
            'QueryNeedToLiquidationPosIds': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryNeedToLiquidationPosIds,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryNeedToLiquidationPosIdsReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryNeedToLiquidatorPosIdsResp.SerializeToString,
            ),
            'QueryAccountNumber': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryAccountNumber,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryAccountNumberReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryAccountNumberResp.SerializeToString,
            ),
            'QueryReserve': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryReserve,
                    request_deserializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryReserveReq.FromString,
                    response_serializer=fx_dot_dex_dot_v1_dot_query__pb2.QueryReserveResp.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fx.dex.v1.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Query defines the gRPC querier service.
    """

    @staticmethod
    def QueryParams(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryParams',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryParamsReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryParamsResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryOrders(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryOrders',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryOrdersRequest.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryOrdersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryOrder(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryOrder',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderRequest.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryOrderbook(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryOrderbook',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderbookReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryOrderbookResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryPositions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryPositions',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionsReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionsResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryPosition(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryPosition',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPositionResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryFunding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryFunding',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryPairFundingRates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryPairFundingRates',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPairFundingRatesReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPairFundingRatesResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryFundingTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryFundingTime',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingTimeReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryFundingTimeResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryDealPrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryDealPrice',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryDealPriceReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryDealPriceResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryMatchResult(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryMatchResult',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMatchResultReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMatchResultResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryMarkPrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryMarkPrice',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkPriceReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkPriceResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryMarkAndOraclePrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryMarkAndOraclePrice',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkAndOraclePriceReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMarkAndOraclePriceResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryLiquidationPrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryLiquidationPrice',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryLiquidationPriceReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryLiquidationPriceResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryMovingAverage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryMovingAverage',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMAReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryMAResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryStoreStatistic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryStoreStatistic',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryStoreStatisticReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryStoreStatisticResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryIsFunding(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryIsFunding',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryIsFundingReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryIsFundingResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryPremiumIndexConf(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryPremiumIndexConf',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPremiumIndexConfReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryPremiumIndexConfResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryNeedToLiquidationPosIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryNeedToLiquidationPosIds',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryNeedToLiquidationPosIdsReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryNeedToLiquidatorPosIdsResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryAccountNumber(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryAccountNumber',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryAccountNumberReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryAccountNumberResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def QueryReserve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.dex.v1.Query/QueryReserve',
            fx_dot_dex_dot_v1_dot_query__pb2.QueryReserveReq.SerializeToString,
            fx_dot_dex_dot_v1_dot_query__pb2.QueryReserveResp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/feegrant/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n cosmos/feegrant/v1beta1/tx.proto\x12\x17\x63osmos.feegrant.v1beta1\x1a\x19google/protobuf/any.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x17\x63osmos/msg/v1/msg.proto\"\xb3\x01\n\x11MsgGrantAllowance\x12)\n\x07granter\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12)\n\x07grantee\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12:\n\tallowance\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyB\x11\xca\xb4-\rFeeAllowanceI:\x0c\x82\xe7\xb0*\x07granter\"\x1b\n\x19MsgGrantAllowanceResponse\"x\n\x12MsgRevokeAllowance\x12)\n\x07granter\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12)\n\x07grantee\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x0c\x82\xe7\xb0*\x07granter\"\x1c\n\x1aMsgRevokeAllowanceResponse2\xec\x01\n\x03Msg\x12p\n\x0eGrantAllowance\x12*.cosmos.feegrant.v1beta1.MsgGrantAllowance\x1a\x32.cosmos.feegrant.v1beta1.MsgGrantAllowanceResponse\x12s\n\x0fRevokeAllowance\x12+.cosmos.feegrant.v1beta1.MsgRevokeAllowance\x1a\x33.cosmos.feegrant.v1beta1.MsgRevokeAllowanceResponseB)Z\'github.com/cosmos/cosmos-sdk/x/feegrantb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.feegrant.v1beta1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\'github.com/cosmos/cosmos-sdk/x/feegrant'
  _MSGGRANTALLOWANCE.fields_by_name['granter']._options = None
  _MSGGRANTALLOWANCE.fields_by_name['granter']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGGRANTALLOWANCE.fields_by_name['grantee']._options = None
  _MSGGRANTALLOWANCE.fields_by_name['grantee']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGGRANTALLOWANCE.fields_by_name['allowance']._options = None
  _MSGGRANTALLOWANCE.fields_by_name['allowance']._serialized_options = b'\312\264-\rFeeAllowanceI'
  _MSGGRANTALLOWANCE._options = None
  _MSGGRANTALLOWANCE._serialized_options = b'\202\347\260*\007granter'
  _MSGREVOKEALLOWANCE.fields_by_name['granter']._options = None
  _MSGREVOKEALLOWANCE.fields_by_name['granter']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGREVOKEALLOWANCE.fields_by_name['grantee']._options = None
  _MSGREVOKEALLOWANCE.fields_by_name['grantee']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGREVOKEALLOWANCE._options = None
  _MSGREVOKEALLOWANCE._serialized_options = b'\202\347\260*\007granter'
  _globals['_MSGGRANTALLOWANCE']._serialized_start=141
  _globals['_MSGGRANTALLOWANCE']._serialized_end=320
  _globals['_MSGGRANTALLOWANCERESPONSE']._serialized_start=322
  _globals['_MSGGRANTALLOWANCERESPONSE']._serialized_end=349
  _globals['_MSGREVOKEALLOWANCE']._serialized_start=351
  _globals['_MSGREVOKEALLOWANCE']._serialized_end=471
  _globals['_MSGREVOKEALLOWANCERESPONSE']._serialized_start=473
  _globals['_MSGREVOKEALLOWANCERESPONSE']._serialized_end=501
  _globals['_MSG']._serialized_start=504
  _globals['_MSG']._serialized_end=740
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/auth/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x66x/auth/v1/query.proto\x12\nfx.auth.v1\x1a\x1cgoogle/api/annotations.proto\"8\n\x15\x43onvertAddressRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0e\n\x06prefix\x18\x02 \x01(\t\")\n\x16\x43onvertAddressResponse\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t2\x86\x01\n\x05Query\x12}\n\x0e\x43onvertAddress\x12!.fx.auth.v1.ConvertAddressRequest\x1a\".fx.auth.v1.ConvertAddressResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/fx/auth/v1/bech32/{address}B/Z-github.com/functionx/fx-core/server/grpc/authb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.auth.v1.query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z-github.com/functionx/fx-core/server/grpc/auth'
  _QUERY.methods_by_name['ConvertAddress']._options = None
  _QUERY.methods_by_name['ConvertAddress']._serialized_options = b'\202\323\344\223\002\036\022\034/fx/auth/v1/bech32/{address}'
  _globals['_CONVERTADDRESSREQUEST']._serialized_start=68
  _globals['_CONVERTADDRESSREQUEST']._serialized_end=124
  _globals['_CONVERTADDRESSRESPONSE']._serialized_start=126
  _globals['_CONVERTADDRESSRESPONSE']._serialized_end=167
  _globals['_QUERY']._serialized_start=170
  _globals['_QUERY']._serialized_end=304
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/dex/v1/funding.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66x/dex/v1/funding.proto\x12\tfx.dex.v1\x1a\x14gogoproto/gogo.proto\"s\n\x0fPairFundingRate\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x39\n\x0c\x66unding_rate\x18\x02 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x14\n\x0c\x66unding_time\x18\x03 \x01(\x03\"\xe3\x01\n\x07\x46unding\x12\x16\n\x0e\x66unding_period\x18\x01 \x01(\x03\x12\x19\n\x11next_funding_time\x18\x02 \x01(\x03\x12\x15\n\rfunding_times\x18\x03 \x01(\x03\x12\x1d\n\x15max_funding_per_block\x18\x04 \x01(\x03\x12#\n\x15\x63rypto_funding_period\x18\x05 \x01(\x03\x42\x04\xc8\xde\x1f\x01\x12&\n\x18\x63rypto_next_funding_time\x18\x06 \x01(\x03\x42\x04\xc8\xde\x1f\x01\x12\"\n\x14\x63rypto_funding_times\x18\x07 \x01(\x03\x42\x04\xc8\xde\x1f\x01\"\x8e\x01\n\x0b\x46undingTime\x12\x13\n\x0bsummer_time\x18\x01 \x01(\x03\x12\x13\n\x0bwinter_time\x18\x02 \x01(\x03\x12\x15\n\rsummer_clocks\x18\x03 \x03(\t\x12\x15\n\rwinter_clocks\x18\x04 \x03(\t\x12\x10\n\x08holidays\x18\x05 \x03(\t\x12\x15\n\rcrypto_clocks\x18\x06 \x03(\t\"x\n\rSettleFunding\x12\x12\n\nis_funding\x18\x01 \x01(\x08\x12\x38\n\x10next_position_id\x18\x02 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\x12\x19\n\x0bmarket_type\x18\x03 \x01(\tB\x04\xc8\xde\x1f\x01\"\xc5\x02\n\x10PremiumIndexConf\x12\x15\n\rupdate_period\x18\x01 \x01(\x03\x12\x18\n\x10next_update_time\x18\x02 \x01(\x03\x12\x18\n\x10round_start_time\x18\x03 \x01(\x03\x12\x17\n\x0fnext_round_time\x18\x04 \x01(\x03\x12\x13\n\x0bround_index\x18\x05 \x01(\x05\x12\"\n\x14\x63rypto_update_period\x18\x06 \x01(\x03\x42\x04\xc8\xde\x1f\x01\x12%\n\x17\x63rypto_next_update_time\x18\x07 \x01(\x03\x42\x04\xc8\xde\x1f\x01\x12%\n\x17\x63rypto_round_start_time\x18\x08 \x01(\x03\x42\x04\xc8\xde\x1f\x01\x12$\n\x16\x63rypto_next_round_time\x18\t \x01(\x03\x42\x04\xc8\xde\x1f\x01\x12 \n\x12\x63rypto_round_index\x18\n \x01(\x05\x42\x04\xc8\xde\x1f\x01\"\xb3\x01\n\x0cPremiumIndex\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x17\n\x0fround_timestamp\x18\x02 \x01(\x03\x12\x31\n\x05ticks\x18\x03 \x03(\x0b\x32\x1c.fx.dex.v1.PremiumIndex.TickB\x04\xc8\xde\x1f\x00\x1a\x46\n\x04Tick\x12\x32\n\x05value\x18\x01 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\n\n\x02ts\x18\x02 \x01(\x03\"F\n\x13MovingAverageParams\x12\x15\n\rupdate_period\x18\x01 \x01(\x03\x12\x18\n\x10next_update_time\x18\x02 \x01(\x03\"K\n\rMovingAverage\x12:\n\raverage_price\x18\x01 \x03(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDecB-Z+github.com/marginxio/marginx/x/dex/v1/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.dex.v1.funding_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/marginxio/marginx/x/dex/v1/types'
  _PAIRFUNDINGRATE.fields_by_name['funding_rate']._options = None
  _PAIRFUNDINGRATE.fields_by_name['funding_rate']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _FUNDING.fields_by_name['crypto_funding_period']._options = None
  _FUNDING.fields_by_name['crypto_funding_period']._serialized_options = b'\310\336\037\001'
  _FUNDING.fields_by_name['crypto_next_funding_time']._options = None
  _FUNDING.fields_by_name['crypto_next_funding_time']._serialized_options = b'\310\336\037\001'
  _FUNDING.fields_by_name['crypto_funding_times']._options = None
  _FUNDING.fields_by_name['crypto_funding_times']._serialized_options = b'\310\336\037\001'
  _SETTLEFUNDING.fields_by_name['next_position_id']._options = None
  _SETTLEFUNDING.fields_by_name['next_position_id']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _SETTLEFUNDING.fields_by_name['market_type']._options = None
  _SETTLEFUNDING.fields_by_name['market_type']._serialized_options = b'\310\336\037\001'
  _PREMIUMINDEXCONF.fields_by_name['crypto_update_period']._options = None
  _PREMIUMINDEXCONF.fields_by_name['crypto_update_period']._serialized_options = b'\310\336\037\001'
  _PREMIUMINDEXCONF.fields_by_name['crypto_next_update_time']._options = None
  _PREMIUMINDEXCONF.fields_by_name['crypto_next_update_time']._serialized_options = b'\310\336\037\001'
  _PREMIUMINDEXCONF.fields_by_name['crypto_round_start_time']._options = None
  _PREMIUMINDEXCONF.fields_by_name['crypto_round_start_time']._serialized_options = b'\310\336\037\001'
  _PREMIUMINDEXCONF.fields_by_name['crypto_next_round_time']._options = None
  _PREMIUMINDEXCONF.fields_by_name['crypto_next_round_time']._serialized_options = b'\310\336\037\001'
  _PREMIUMINDEXCONF.fields_by_name['crypto_round_index']._options = None
  _PREMIUMINDEXCONF.fields_by_name['crypto_round_index']._serialized_options = b'\310\336\037\001'
  _PREMIUMINDEX_TICK.fields_by_name['value']._options = None
  _PREMIUMINDEX_TICK.fields_by_name['value']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _PREMIUMINDEX.fields_by_name['ticks']._options = None
  _PREMIUMINDEX.fields_by_name['ticks']._serialized_options = b'\310\336\037\000'
  _MOVINGAVERAGE.fields_by_name['average_price']._options = None
  _MOVINGAVERAGE.fields_by_name['average_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _globals['_PAIRFUNDINGRATE']._serialized_start=60
  _globals['_PAIRFUNDINGRATE']._serialized_end=175
  _globals['_FUNDING']._serialized_start=178
  _globals['_FUNDING']._serialized_end=405
  _globals['_FUNDINGTIME']._serialized_start=408
  _globals['_FUNDINGTIME']._serialized_end=550
  _globals['_SETTLEFUNDING']._serialized_start=552
  _globals['_SETTLEFUNDING']._serialized_end=672
  _globals['_PREMIUMINDEXCONF']._serialized_start=675
  _globals['_PREMIUMINDEXCONF']._serialized_end=1000
  _globals['_PREMIUMINDEX']._serialized_start=1003
  _globals['_PREMIUMINDEX']._serialized_end=1182
  _globals['_PREMIUMINDEX_TICK']._serialized_start=1112
  _globals['_PREMIUMINDEX_TICK']._serialized_end=1182
  _globals['_MOVINGAVERAGEPARAMS']._serialized_start=1184
  _globals['_MOVINGAVERAGEPARAMS']._serialized_end=1254
  _globals['_MOVINGAVERAGE']._serialized_start=1256
  _globals['_MOVINGAVERAGE']._serialized_end=1331
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/slashing/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n cosmos/slashing/v1beta1/tx.proto\x12\x17\x63osmos.slashing.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x17\x63osmos/msg/v1/msg.proto\"e\n\tMsgUnjail\x12;\n\x0evalidator_addr\x18\x01 \x01(\tB#\xea\xde\x1f\x07\x61\x64\x64ress\xd2\xb4-\x14\x63osmos.AddressString:\x1b\x88\xa0\x1f\x00\x98\xa0\x1f\x01\x82\xe7\xb0*\x0evalidator_addr\"\x13\n\x11MsgUnjailResponse2_\n\x03Msg\x12X\n\x06Unjail\x12\".cosmos.slashing.v1beta1.MsgUnjail\x1a*.cosmos.slashing.v1beta1.MsgUnjailResponseB3Z-github.com/cosmos/cosmos-sdk/x/slashing/types\xa8\xe2\x1e\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.slashing.v1beta1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z-github.com/cosmos/cosmos-sdk/x/slashing/types\250\342\036\001'
  _MSGUNJAIL.fields_by_name['validator_addr']._options = None
  _MSGUNJAIL.fields_by_name['validator_addr']._serialized_options = b'\352\336\037\007address\322\264-\024cosmos.AddressString'
  _MSGUNJAIL._options = None
  _MSGUNJAIL._serialized_options = b'\210\240\037\000\230\240\037\001\202\347\260*\016validator_addr'
  _globals['_MSGUNJAIL']._serialized_start=135
  _globals['_MSGUNJAIL']._serialized_end=236
  _globals['_MSGUNJAILRESPONSE']._serialized_start=238
  _globals['_MSGUNJAILRESPONSE']._serialized_end=257
  _globals['_MSG']._serialized_start=259
  _globals['_MSG']._serialized_end=354
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/distribution/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$cosmos/distribution/v1beta1/tx.proto\x12\x1b\x63osmos.distribution.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x17\x63osmos/msg/v1/msg.proto\"\xa0\x01\n\x15MsgSetWithdrawAddress\x12\x33\n\x11\x64\x65legator_address\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x32\n\x10withdraw_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x1e\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x11\x64\x65legator_address\"\x1f\n\x1dMsgSetWithdrawAddressResponse\"\xa6\x01\n\x1aMsgWithdrawDelegatorReward\x12\x33\n\x11\x64\x65legator_address\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x33\n\x11validator_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x1e\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x11\x64\x65legator_address\"\x81\x01\n\"MsgWithdrawDelegatorRewardResponse\x12[\n\x06\x61mount\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\"u\n\x1eMsgWithdrawValidatorCommission\x12\x33\n\x11validator_address\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x1e\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x11validator_address\"\x85\x01\n&MsgWithdrawValidatorCommissionResponse\x12[\n\x06\x61mount\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\"\xb8\x01\n\x14MsgFundCommunityPool\x12[\n\x06\x61mount\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\x12+\n\tdepositor\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x16\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\tdepositor\"\x1e\n\x1cMsgFundCommunityPoolResponse2\xc8\x04\n\x03Msg\x12\x84\x01\n\x12SetWithdrawAddress\x12\x32.cosmos.distribution.v1beta1.MsgSetWithdrawAddress\x1a:.cosmos.distribution.v1beta1.MsgSetWithdrawAddressResponse\x12\x93\x01\n\x17WithdrawDelegatorReward\x12\x37.cosmos.distribution.v1beta1.MsgWithdrawDelegatorReward\x1a?.cosmos.distribution.v1beta1.MsgWithdrawDelegatorRewardResponse\x12\x9f\x01\n\x1bWithdrawValidatorCommission\x12;.cosmos.distribution.v1beta1.MsgWithdrawValidatorCommission\x1a\x43.cosmos.distribution.v1beta1.MsgWithdrawValidatorCommissionResponse\x12\x81\x01\n\x11\x46undCommunityPool\x12\x31.cosmos.distribution.v1beta1.MsgFundCommunityPool\x1a\x39.cosmos.distribution.v1beta1.MsgFundCommunityPoolResponseB7Z1github.com/cosmos/cosmos-sdk/x/distribution/types\xa8\xe2\x1e\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.distribution.v1beta1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z1github.com/cosmos/cosmos-sdk/x/distribution/types\250\342\036\001'
  _MSGSETWITHDRAWADDRESS.fields_by_name['delegator_address']._options = None
  _MSGSETWITHDRAWADDRESS.fields_by_name['delegator_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGSETWITHDRAWADDRESS.fields_by_name['withdraw_address']._options = None
  _MSGSETWITHDRAWADDRESS.fields_by_name['withdraw_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGSETWITHDRAWADDRESS._options = None
  _MSGSETWITHDRAWADDRESS._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\021delegator_address'
  _MSGWITHDRAWDELEGATORREWARD.fields_by_name['delegator_address']._options = None
  _MSGWITHDRAWDELEGATORREWARD.fields_by_name['delegator_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGWITHDRAWDELEGATORREWARD.fields_by_name['validator_address']._options = None
  _MSGWITHDRAWDELEGATORREWARD.fields_by_name['validator_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGWITHDRAWDELEGATORREWARD._options = None
  _MSGWITHDRAWDELEGATORREWARD._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\021delegator_address'
  _MSGWITHDRAWDELEGATORREWARDRESPONSE.fields_by_name['amount']._options = None
  _MSGWITHDRAWDELEGATORREWARDRESPONSE.fields_by_name['amount']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _MSGWITHDRAWVALIDATORCOMMISSION.fields_by_name['validator_address']._options = None
  _MSGWITHDRAWVALIDATORCOMMISSION.fields_by_name['validator_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGWITHDRAWVALIDATORCOMMISSION._options = None
  _MSGWITHDRAWVALIDATORCOMMISSION._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\021validator_address'
  _MSGWITHDRAWVALIDATORCOMMISSIONRESPONSE.fields_by_name['amount']._options = None
  _MSGWITHDRAWVALIDATORCOMMISSIONRESPONSE.fields_by_name['amount']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _MSGFUNDCOMMUNITYPOOL.fields_by_name['amount']._options = None
  _MSGFUNDCOMMUNITYPOOL.fields_by_name['amount']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _MSGFUNDCOMMUNITYPOOL.fields_by_name['depositor']._options = None
  _MSGFUNDCOMMUNITYPOOL.fields_by_name['depositor']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGFUNDCOMMUNITYPOOL._options = None
  _MSGFUNDCOMMUNITYPOOL._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\tdepositor'
  _globals['_MSGSETWITHDRAWADDRESS']._serialized_start=176
  _globals['_MSGSETWITHDRAWADDRESS']._serialized_end=336
  _globals['_MSGSETWITHDRAWADDRESSRESPONSE']._serialized_start=338
  _globals['_MSGSETWITHDRAWADDRESSRESPONSE']._serialized_end=369
  _globals['_MSGWITHDRAWDELEGATORREWARD']._serialized_start=372
  _globals['_MSGWITHDRAWDELEGATORREWARD']._serialized_end=538
  _globals['_MSGWITHDRAWDELEGATORREWARDRESPONSE']._serialized_start=541
  _globals['_MSGWITHDRAWDELEGATORREWARDRESPONSE']._serialized_end=670
  _globals['_MSGWITHDRAWVALIDATORCOMMISSION']._serialized_start=672
  _globals['_MSGWITHDRAWVALIDATORCOMMISSION']._serialized_end=789
  _globals['_MSGWITHDRAWVALIDATORCOMMISSIONRESPONSE']._serialized_start=792
  _globals['_MSGWITHDRAWVALIDATORCOMMISSIONRESPONSE']._serialized_end=925
  _globals['_MSGFUNDCOMMUNITYPOOL']._serialized_start=928
  _globals['_MSGFUNDCOMMUNITYPOOL']._serialized_end=1112
  _globals['_MSGFUNDCOMMUNITYPOOLRESPONSE']._serialized_start=1114
  _globals['_MSGFUNDCOMMUNITYPOOLRESPONSE']._serialized_end=1144
  _globals['_MSG']._serialized_start=1147
  _globals['_MSG']._serialized_end=1731
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/legacy/gravity/v1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.fx.legacy.gravity.v1 import tx_pb2 as fx_dot_legacy_dot_gravity_dot_v1_dot_tx__pb2
from fxsdk.x.fx.legacy.gravity.v1 import types_pb2 as fx_dot_legacy_dot_gravity_dot_v1_dot_types__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"fx/legacy/gravity/v1/genesis.proto\x12\rfx.gravity.v1\x1a\x1d\x66x/legacy/gravity/v1/tx.proto\x1a fx/legacy/gravity/v1/types.proto\x1a\x14gogoproto/gogo.proto\"\x99\x06\n\x06Params\x12\x12\n\ngravity_id\x18\x01 \x01(\t\x12\x1c\n\x14\x63ontract_source_hash\x18\x02 \x01(\t\x12\x1a\n\x12\x62ridge_eth_address\x18\x04 \x01(\t\x12\x17\n\x0f\x62ridge_chain_id\x18\x05 \x01(\x04\x12\x1d\n\x15signed_valsets_window\x18\x06 \x01(\x04\x12\x1d\n\x15signed_batches_window\x18\x07 \x01(\x04\x12\x1c\n\x14signed_claims_window\x18\x08 \x01(\x04\x12\x1c\n\x14target_batch_timeout\x18\n \x01(\x04\x12\x1a\n\x12\x61verage_block_time\x18\x0b \x01(\x04\x12\x1e\n\x16\x61verage_eth_block_time\x18\x0c \x01(\x04\x12M\n\x15slash_fraction_valset\x18\r \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12L\n\x14slash_fraction_batch\x18\x0e \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12L\n\x14slash_fraction_claim\x18\x0f \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12X\n slash_fraction_conflicting_claim\x18\x10 \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12&\n\x1eunbond_slashing_valsets_window\x18\x11 \x01(\x04\x12#\n\x1bibc_transfer_timeout_height\x18\x12 \x01(\x04\x12Z\n\"valset_update_power_change_percent\x18\x13 \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x04\x80\xdc \x00\"\xc6\x06\n\x0cGenesisState\x12+\n\x06params\x18\x01 \x01(\x0b\x32\x15.fx.gravity.v1.ParamsB\x04\xc8\xde\x1f\x00\x12\x1b\n\x13last_observed_nonce\x18\x02 \x01(\x04\x12X\n\x1alast_observed_block_height\x18\x03 \x01(\x0b\x32..fx.gravity.v1.LastObservedEthereumBlockHeightB\x04\xc8\xde\x1f\x00\x12\x45\n\rdelegate_keys\x18\x04 \x03(\x0b\x32(.fx.gravity.v1.MsgSetOrchestratorAddressB\x04\xc8\xde\x1f\x00\x12,\n\x07valsets\x18\x05 \x03(\x0b\x32\x15.fx.gravity.v1.ValsetB\x04\xc8\xde\x1f\x00\x12:\n\x0f\x65rc20_to_denoms\x18\x06 \x03(\x0b\x32\x1b.fx.gravity.v1.ERC20ToDenomB\x04\xc8\xde\x1f\x00\x12\x44\n\x13unbatched_transfers\x18\x07 \x03(\x0b\x32!.fx.gravity.v1.OutgoingTransferTxB\x04\xc8\xde\x1f\x00\x12\x35\n\x07\x62\x61tches\x18\x08 \x03(\x0b\x32\x1e.fx.gravity.v1.OutgoingTxBatchB\x04\xc8\xde\x1f\x00\x12<\n\x0e\x62\x61tch_confirms\x18\t \x03(\x0b\x32\x1e.fx.gravity.v1.MsgConfirmBatchB\x04\xc8\xde\x1f\x00\x12>\n\x0fvalset_confirms\x18\n \x03(\x0b\x32\x1f.fx.gravity.v1.MsgValsetConfirmB\x04\xc8\xde\x1f\x00\x12\x36\n\x0c\x61ttestations\x18\x0b \x03(\x0b\x32\x1a.fx.gravity.v1.AttestationB\x04\xc8\xde\x1f\x00\x12\x39\n\x14last_observed_valset\x18\x0c \x01(\x0b\x32\x15.fx.gravity.v1.ValsetB\x04\xc8\xde\x1f\x00\x12 \n\x18last_slashed_batch_block\x18\r \x01(\x04\x12!\n\x19last_slashed_valset_nonce\x18\x0e \x01(\x04\x12\x17\n\x0flast_tx_pool_id\x18\x0f \x01(\x04\x12\x15\n\rlast_batch_id\x18\x10 \x01(\x04\x42.Z,github.com/functionx/fx-core/x/gravity/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.legacy.gravity.v1.genesis_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/functionx/fx-core/x/gravity/types'
  _PARAMS.fields_by_name['slash_fraction_valset']._options = None
  _PARAMS.fields_by_name['slash_fraction_valset']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_batch']._options = None
  _PARAMS.fields_by_name['slash_fraction_batch']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_claim']._options = None
  _PARAMS.fields_by_name['slash_fraction_claim']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_conflicting_claim']._options = None
  _PARAMS.fields_by_name['slash_fraction_conflicting_claim']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['valset_update_power_change_percent']._options = None
  _PARAMS.fields_by_name['valset_update_power_change_percent']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS._options = None
  _PARAMS._serialized_options = b'\200\334 \000'
  _GENESISSTATE.fields_by_name['params']._options = None
  _GENESISSTATE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['last_observed_block_height']._options = None
  _GENESISSTATE.fields_by_name['last_observed_block_height']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['delegate_keys']._options = None
  _GENESISSTATE.fields_by_name['delegate_keys']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['valsets']._options = None
  _GENESISSTATE.fields_by_name['valsets']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['erc20_to_denoms']._options = None
  _GENESISSTATE.fields_by_name['erc20_to_denoms']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['unbatched_transfers']._options = None
  _GENESISSTATE.fields_by_name['unbatched_transfers']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['batches']._options = None
  _GENESISSTATE.fields_by_name['batches']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['batch_confirms']._options = None
  _GENESISSTATE.fields_by_name['batch_confirms']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['valset_confirms']._options = None
  _GENESISSTATE.fields_by_name['valset_confirms']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['attestations']._options = None
  _GENESISSTATE.fields_by_name['attestations']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['last_observed_valset']._options = None
  _GENESISSTATE.fields_by_name['last_observed_valset']._serialized_options = b'\310\336\037\000'
  _globals['_PARAMS']._serialized_start=141
  _globals['_PARAMS']._serialized_end=934
  _globals['_GENESISSTATE']._serialized_start=937
  _globals['_GENESISSTATE']._serialized_end=1775
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/migrate/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x66x/migrate/v1/tx.proto\x12\rfx.migrate.v1\"@\n\x11MsgMigrateAccount\x12\x0c\n\x04\x66rom\x18\x01 \x01(\t\x12\n\n\x02to\x18\x02 \x01(\t\x12\x11\n\tsignature\x18\x03 \x01(\t\"\x1b\n\x19MsgMigrateAccountResponse2c\n\x03Msg\x12\\\n\x0eMigrateAccount\x12 .fx.migrate.v1.MsgMigrateAccount\x1a(.fx.migrate.v1.MsgMigrateAccountResponseB.Z,github.com/functionx/fx-core/x/migrate/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.migrate.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/functionx/fx-core/x/migrate/types'
  _globals['_MSGMIGRATEACCOUNT']._serialized_start=41
  _globals['_MSGMIGRATEACCOUNT']._serialized_end=105
  _globals['_MSGMIGRATEACCOUNTRESPONSE']._serialized_start=107
  _globals['_MSGMIGRATEACCOUNTRESPONSE']._serialized_end=134
  _globals['_MSG']._serialized_start=136
  _globals['_MSG']._serialized_end=235
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from fxsdk.x.ethermint.feemarket.v1 import query_pb2 as ethermint_dot_feemarket_dot_v1_dot_query__pb2


class QueryStub(object):
    """Query defines the gRPC querier service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Params = channel.unary_unary(
                '/ethermint.feemarket.v1.Query/Params',
                request_serializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryParamsRequest.SerializeToString,
                response_deserializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryParamsResponse.FromString,
                )
        self.BaseFee = channel.unary_unary(
                '/ethermint.feemarket.v1.Query/BaseFee',
                request_serializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBaseFeeRequest.SerializeToString,
                response_deserializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBaseFeeResponse.FromString,
                )
        self.BlockGas = channel.unary_unary(
                '/ethermint.feemarket.v1.Query/BlockGas',
                request_serializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBlockGasRequest.SerializeToString,
                response_deserializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBlockGasResponse.FromString,
                )


class QueryServicer(object):
    """Query defines the gRPC querier service.
    """

    def Params(self, request, context):
        """Params queries the parameters of x/feemarket module.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BaseFee(self, request, context):
        """BaseFee queries the base fee of the parent block of the current block.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BlockGas(self, request, context):
        """BlockGas queries the gas used at a given block height
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Params': grpc.unary_unary_rpc_method_handler(
                    servicer.Params,
                    request_deserializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryParamsRequest.FromString,
                    response_serializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryParamsResponse.SerializeToString,
            ),
            'BaseFee': grpc.unary_unary_rpc_method_handler(
                    servicer.BaseFee,
                    request_deserializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBaseFeeRequest.FromString,
                    response_serializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBaseFeeResponse.SerializeToString,
            ),
            'BlockGas': grpc.unary_unary_rpc_method_handler(
                    servicer.BlockGas,
                    request_deserializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBlockGasRequest.FromString,
                    response_serializer=ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBlockGasResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ethermint.feemarket.v1.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Query defines the gRPC querier service.
    """

    @staticmethod
    def Params(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ethermint.feemarket.v1.Query/Params',
            ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryParamsRequest.SerializeToString,
            ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryParamsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BaseFee(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ethermint.feemarket.v1.Query/BaseFee',
            ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBaseFeeRequest.SerializeToString,
            ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBaseFeeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BlockGas(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ethermint.feemarket.v1.Query/BlockGas',
            ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBlockGasRequest.SerializeToString,
            ethermint_dot_feemarket_dot_v1_dot_query__pb2.QueryBlockGasResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from decimal import Decimal

DEFAULT_DEC = 1000000


def dec_to_str(amount: Decimal) -> str:
    amount = amount * Decimal(DEFAULT_DEC)
    amount = str(amount)
    return amount.split('.', 1)[0]


def dec_from_str(amount: str) -> Decimal:
    amount = Decimal(amount) / Decimal(DEFAULT_DEC)
    return amount



import asyncio
import logging
import traceback
from random import random
from typing import Optional, Dict

import ujson as json
import websockets.client


class ReconnectingWebsocket:
    MAX_RECONNECTS: int = 5
    MAX_RECONNECT_SECONDS: int = 60
    MIN_RECONNECT_WAIT = 0.1
    TIMEOUT: int = 10
    PROTOCOL_VERSION: str = '1.0.0'

    def __init__(self, endpoint_url: str, loop, recv, retry_count: int = 5):
        self._loop = loop
        self._log = logging.getLogger(__name__)
        self._recv = recv
        self._reconnect_attempts: int = 0
        self._conn = None
        self._endpoint_url = endpoint_url
        self._connect_id: int = -1
        self._ping_timeout = 60
        self._retry_count: int = retry_count
        self._socket: Optional[websockets.client.WebSocketClientProtocol] = None

        self._connect()

    def _connect(self):
        self._conn = asyncio.ensure_future(self._run())

    def _get_ws_endpoint_url(self):
        return f"{self._endpoint_url}/websocket"

    async def _run(self):

        keep_waiting: bool = True

        self._log.info(f"connecting to {self._get_ws_endpoint_url()}")
        try:
            async with websockets.client.connect(self._get_ws_endpoint_url(), loop=self._loop) as socket:
                self._on_connect(socket)

                try:
                    while keep_waiting:
                        try:
                            evt = await asyncio.wait_for(self._socket.recv(), timeout=self._ping_timeout)
                        except asyncio.TimeoutError:
                            self._log.debug("no message in {} seconds".format(self._ping_timeout))
                            await self.send_keepalive()
                        except asyncio.CancelledError:
                            self._log.debug("cancelled error")
                            await self.ping()
                        else:
                            try:
                                evt_obj = json.loads(evt)
                                await self._recv(evt_obj)
                            except ValueError:
                                self._log.error("invalid json: {}".format(evt))
                except websockets.ConnectionClosed as e:
                    self._log.debug('conn closed:{}'.format(e))
                    await self._reconnect()
                except Exception as e:
                    self._log.debug('ws exception:{}'.format(e))
                    await self._reconnect()
        except Exception as e:
            self._log.info(f"websocket error: {e}")
            traceback.print_stack()

    def _on_connect(self, socket):
        self._socket = socket
        self._reconnect_attempts = 0

    async def _reconnect(self):
        await self.cancel()
        self._reconnect_attempts += 1
        if self._reconnect_attempts < self.MAX_RECONNECTS:

            self._log.debug(f"websocket reconnecting {self.MAX_RECONNECTS - self._reconnect_attempts} attempts left")
            reconnect_wait = self._get_reconnect_wait(self._reconnect_attempts)
            self._log.debug(f' waiting {reconnect_wait}')
            await asyncio.sleep(reconnect_wait)
            self._connect()
        else:
            # maybe raise an exception
            self._log.error(f"websocket could not reconnect after {self._reconnect_attempts} attempts")
            pass

    def _get_reconnect_wait(self, attempts: int) -> int:
        expo = 2 ** attempts
        return round(random() * min(self.MAX_RECONNECT_SECONDS, expo - 1) + 1)

    async def send_keepalive(self):
        msg = {"method": "keepAlive"}
        await self._socket.send(json.dumps(msg, ensure_ascii=False))

    async def send_message(self, msg, retry_count=0):
        if not self._socket:
            if retry_count < self._retry_count:
                await asyncio.sleep(1)
                await self.send_message(msg, retry_count + 1)
            else:
                self._log.info("Unable to send, not connected")
        else:
            await self._socket.send(json.dumps(msg, ensure_ascii=False))

    async def ping(self):
        await self._socket.ping()

    async def cancel(self):
        try:
            self._conn.cancel()
        except asyncio.CancelledError:
            pass


class WebsocketManagerBase:

    def __init__(self):
        self._conn = None
        self._log = logging.getLogger(__name__)

    @classmethod
    async def create(cls, endpoint_url: str, loop, callback=None):
        """Create a WebsocketManagerBase instance

        :param endpoint_url: node endpoint url
        :param loop: asyncio loop
        :param callback: async callback function to receive messages
        :return:
        """
        self = WebsocketManagerBase()
        loop = loop
        callback = callback if callback else self.receive
        self._conn = ReconnectingWebsocket(endpoint_url, recv=callback, loop=loop)
        return self

    async def receive(self, data: Dict):
        self._log.debug(f"received data: {data}")

    async def close(self):
        await self._conn.cancel()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ethermint/types/v1/dynamic_fee.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$ethermint/types/v1/dynamic_fee.proto\x12\x12\x65thermint.types.v1\x1a\x14gogoproto/gogo.proto\"i\n\x1b\x45xtensionOptionDynamicFeeTx\x12J\n\x12max_priority_price\x18\x01 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.IntB\"Z github.com/evmos/ethermint/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ethermint.types.v1.dynamic_fee_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z github.com/evmos/ethermint/types'
  _EXTENSIONOPTIONDYNAMICFEETX.fields_by_name['max_priority_price']._options = None
  _EXTENSIONOPTIONDYNAMICFEETX.fields_by_name['max_priority_price']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _globals['_EXTENSIONOPTIONDYNAMICFEETX']._serialized_start=82
  _globals['_EXTENSIONOPTIONDYNAMICFEETX']._serialized_end=187
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/legacy/gravity/v1/types.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n fx/legacy/gravity/v1/types.proto\x12\rfx.gravity.v1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\"5\n\x0f\x42ridgeValidator\x12\r\n\x05power\x18\x01 \x01(\x04\x12\x13\n\x0b\x65th_address\x18\x02 \x01(\t\"X\n\x06Valset\x12\r\n\x05nonce\x18\x01 \x01(\x04\x12/\n\x07members\x18\x02 \x03(\x0b\x32\x1e.fx.gravity.v1.BridgeValidator\x12\x0e\n\x06height\x18\x03 \x01(\x04\"T\n\x1fLastObservedEthereumBlockHeight\x12\x17\n\x0f\x66x_block_height\x18\x01 \x01(\x04\x12\x18\n\x10\x65th_block_height\x18\x02 \x01(\x04\",\n\x0c\x45RC20ToDenom\x12\r\n\x05\x65rc20\x18\x01 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x02 \x01(\t\"^\n\nERC20Token\x12\x10\n\x08\x63ontract\x18\x01 \x01(\t\x12>\n\x06\x61mount\x18\x02 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\"c\n\x0b\x41ttestation\x12\x10\n\x08observed\x18\x01 \x01(\x08\x12\r\n\x05votes\x18\x02 \x03(\t\x12\x0e\n\x06height\x18\x03 \x01(\x04\x12#\n\x05\x63laim\x18\x04 \x01(\x0b\x32\x14.google.protobuf.Any\"\xb1\x01\n\x0fOutgoingTxBatch\x12\x13\n\x0b\x62\x61tch_nonce\x18\x01 \x01(\x04\x12\x15\n\rbatch_timeout\x18\x02 \x01(\x04\x12\x37\n\x0ctransactions\x18\x03 \x03(\x0b\x32!.fx.gravity.v1.OutgoingTransferTx\x12\x16\n\x0etoken_contract\x18\x04 \x01(\t\x12\r\n\x05\x62lock\x18\x05 \x01(\x04\x12\x12\n\nfeeReceive\x18\x06 \x01(\t\"\xa4\x01\n\x12OutgoingTransferTx\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0e\n\x06sender\x18\x02 \x01(\t\x12\x14\n\x0c\x64\x65st_address\x18\x03 \x01(\t\x12.\n\x0b\x65rc20_token\x18\x04 \x01(\x0b\x32\x19.fx.gravity.v1.ERC20Token\x12,\n\terc20_fee\x18\x05 \x01(\x0b\x32\x19.fx.gravity.v1.ERC20Token\"\xc0\x01\n\tBatchFees\x12\x16\n\x0etoken_contract\x18\x01 \x01(\t\x12\x42\n\ntotal_fees\x18\x02 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\x12\x11\n\ttotal_txs\x18\x03 \x01(\x04\x12\x44\n\x0ctotal_amount\x18\x04 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\"f\n\x0bMinBatchFee\x12\x16\n\x0etoken_contract\x18\x01 \x01(\t\x12?\n\x07\x62\x61seFee\x18\x02 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int*\x9e\x01\n\tClaimType\x12\x1a\n\x16\x43LAIM_TYPE_UNSPECIFIED\x10\x00\x12\x16\n\x12\x43LAIM_TYPE_DEPOSIT\x10\x01\x12\x17\n\x13\x43LAIM_TYPE_WITHDRAW\x10\x02\x12\x1f\n\x1b\x43LAIM_TYPE_ORIGINATED_TOKEN\x10\x03\x12\x1d\n\x19\x43LAIM_TYPE_VALSET_UPDATED\x10\x04\x1a\x04\x88\xa3\x1e\x00\x42.Z,github.com/functionx/fx-core/x/gravity/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.legacy.gravity.v1.types_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/functionx/fx-core/x/gravity/types'
  _CLAIMTYPE._options = None
  _CLAIMTYPE._serialized_options = b'\210\243\036\000'
  _ERC20TOKEN.fields_by_name['amount']._options = None
  _ERC20TOKEN.fields_by_name['amount']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _BATCHFEES.fields_by_name['total_fees']._options = None
  _BATCHFEES.fields_by_name['total_fees']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _BATCHFEES.fields_by_name['total_amount']._options = None
  _BATCHFEES.fields_by_name['total_amount']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _MINBATCHFEE.fields_by_name['baseFee']._options = None
  _MINBATCHFEE.fields_by_name['baseFee']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _globals['_CLAIMTYPE']._serialized_start=1221
  _globals['_CLAIMTYPE']._serialized_end=1379
  _globals['_BRIDGEVALIDATOR']._serialized_start=100
  _globals['_BRIDGEVALIDATOR']._serialized_end=153
  _globals['_VALSET']._serialized_start=155
  _globals['_VALSET']._serialized_end=243
  _globals['_LASTOBSERVEDETHEREUMBLOCKHEIGHT']._serialized_start=245
  _globals['_LASTOBSERVEDETHEREUMBLOCKHEIGHT']._serialized_end=329
  _globals['_ERC20TODENOM']._serialized_start=331
  _globals['_ERC20TODENOM']._serialized_end=375
  _globals['_ERC20TOKEN']._serialized_start=377
  _globals['_ERC20TOKEN']._serialized_end=471
  _globals['_ATTESTATION']._serialized_start=473
  _globals['_ATTESTATION']._serialized_end=572
  _globals['_OUTGOINGTXBATCH']._serialized_start=575
  _globals['_OUTGOINGTXBATCH']._serialized_end=752
  _globals['_OUTGOINGTRANSFERTX']._serialized_start=755
  _globals['_OUTGOINGTRANSFERTX']._serialized_end=919
  _globals['_BATCHFEES']._serialized_start=922
  _globals['_BATCHFEES']._serialized_end=1114
  _globals['_MINBATCHFEE']._serialized_start=1116
  _globals['_MINBATCHFEE']._serialized_end=1218
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: marginx/oracle/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.marginx.oracle.v1 import types_pb2 as marginx_dot_oracle_dot_v1_dot_types__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1amarginx/oracle/v1/tx.proto\x12\x11marginx.oracle.v1\x1a\x14gogoproto/gogo.proto\x1a\x1dmarginx/oracle/v1/types.proto\x1a\x17\x63osmos/msg/v1/msg.proto\x1a\x19\x63osmos_proto/cosmos.proto\"\x86\x01\n\x1bMsgBatchSubmitOracleAnswers\x12\x14\n\x0c\x66rom_address\x18\x01 \x01(\t\x12\x36\n\x07\x61nswers\x18\x02 \x03(\x0b\x32\x1f.marginx.oracle.v1.OracleAnswerB\x04\xc8\xde\x1f\x00:\x19\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x0c\x66rom_address\"%\n#MsgBatchSubmitOracleAnswersResponse\"\x84\x01\n\x0cOracleAnswer\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0e\n\x06height\x18\x02 \x01(\x03\x12\x41\n\x06\x61nswer\x18\x03 \x01(\tB1\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\xd2\xb4-\ncosmos.Dec\x12\x11\n\tsignature\x18\x04 \x01(\x0c\"\x93\x01\n\x13MsgUpdateAggregator\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x37\n\naggregator\x18\x02 \x01(\x0b\x32\x1d.marginx.oracle.v1.AggregatorB\x04\xc8\xde\x1f\x00:\x16\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\tauthority\"\x1d\n\x1bMsgUpdateAggregatorResponse\"\xea\x01\n\x0fMsgPresetOracle\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x30\n\x07markets\x18\x02 \x03(\x0b\x32\x19.marginx.oracle.v1.MarketB\x04\xc8\xde\x1f\x00\x12.\n\x06prices\x18\x03 \x03(\x0b\x32\x18.marginx.oracle.v1.PriceB\x04\xc8\xde\x1f\x00\x12\x38\n\x0b\x61ggregators\x18\x04 \x03(\x0b\x32\x1d.marginx.oracle.v1.AggregatorB\x04\xc8\xde\x1f\x00:\x0e\x82\xe7\xb0*\tauthority\"\x19\n\x17MsgPresetOracleResponse2\xd6\x02\n\x03Msg\x12\x82\x01\n\x18\x42\x61tchSubmitOracleAnswers\x12..marginx.oracle.v1.MsgBatchSubmitOracleAnswers\x1a\x36.marginx.oracle.v1.MsgBatchSubmitOracleAnswersResponse\x12j\n\x10UpdateAggregator\x12&.marginx.oracle.v1.MsgUpdateAggregator\x1a..marginx.oracle.v1.MsgUpdateAggregatorResponse\x12^\n\x0cPresetOracle\x12\".marginx.oracle.v1.MsgPresetOracle\x1a*.marginx.oracle.v1.MsgPresetOracleResponseB-Z+github.com/marginxio/marginx/x/oracle/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'marginx.oracle.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/marginxio/marginx/x/oracle/types'
  _MSGBATCHSUBMITORACLEANSWERS.fields_by_name['answers']._options = None
  _MSGBATCHSUBMITORACLEANSWERS.fields_by_name['answers']._serialized_options = b'\310\336\037\000'
  _MSGBATCHSUBMITORACLEANSWERS._options = None
  _MSGBATCHSUBMITORACLEANSWERS._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\014from_address'
  _ORACLEANSWER.fields_by_name['answer']._options = None
  _ORACLEANSWER.fields_by_name['answer']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec\322\264-\ncosmos.Dec'
  _MSGUPDATEAGGREGATOR.fields_by_name['authority']._options = None
  _MSGUPDATEAGGREGATOR.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEAGGREGATOR.fields_by_name['aggregator']._options = None
  _MSGUPDATEAGGREGATOR.fields_by_name['aggregator']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEAGGREGATOR._options = None
  _MSGUPDATEAGGREGATOR._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\tauthority'
  _MSGPRESETORACLE.fields_by_name['authority']._options = None
  _MSGPRESETORACLE.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGPRESETORACLE.fields_by_name['markets']._options = None
  _MSGPRESETORACLE.fields_by_name['markets']._serialized_options = b'\310\336\037\000'
  _MSGPRESETORACLE.fields_by_name['prices']._options = None
  _MSGPRESETORACLE.fields_by_name['prices']._serialized_options = b'\310\336\037\000'
  _MSGPRESETORACLE.fields_by_name['aggregators']._options = None
  _MSGPRESETORACLE.fields_by_name['aggregators']._serialized_options = b'\310\336\037\000'
  _MSGPRESETORACLE._options = None
  _MSGPRESETORACLE._serialized_options = b'\202\347\260*\tauthority'
  _globals['_MSGBATCHSUBMITORACLEANSWERS']._serialized_start=155
  _globals['_MSGBATCHSUBMITORACLEANSWERS']._serialized_end=289
  _globals['_MSGBATCHSUBMITORACLEANSWERSRESPONSE']._serialized_start=291
  _globals['_MSGBATCHSUBMITORACLEANSWERSRESPONSE']._serialized_end=328
  _globals['_ORACLEANSWER']._serialized_start=331
  _globals['_ORACLEANSWER']._serialized_end=463
  _globals['_MSGUPDATEAGGREGATOR']._serialized_start=466
  _globals['_MSGUPDATEAGGREGATOR']._serialized_end=613
  _globals['_MSGUPDATEAGGREGATORRESPONSE']._serialized_start=615
  _globals['_MSGUPDATEAGGREGATORRESPONSE']._serialized_end=644
  _globals['_MSGPRESETORACLE']._serialized_start=647
  _globals['_MSGPRESETORACLE']._serialized_end=881
  _globals['_MSGPRESETORACLERESPONSE']._serialized_start=883
  _globals['_MSGPRESETORACLERESPONSE']._serialized_end=908
  _globals['_MSG']._serialized_start=911
  _globals['_MSG']._serialized_end=1253
# @@protoc_insertion_point(module_scope)



from google.protobuf.any_pb2 import Any

from fxsdk.wallet.address import DEFAULT_BECH32_PREFIX, Address
from fxsdk.wallet.key import PrivateKey, ETH_DERIVATION_PATH, mnemonic_to_privkey, ETHSECP256K1_KEY_TYPE
from fxsdk.x.cosmos.base.v1beta1.coin_pb2 import Coin
from fxsdk.x.cosmos.tx.signing.v1beta1.signing_pb2 import SIGN_MODE_DIRECT
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import AuthInfo
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import Fee
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import ModeInfo
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import SignDoc
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import SignerInfo
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import Tx
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import TxBody


class TxBuilder:
    def __init__(self, private_key: PrivateKey,
                 chain_id: str = '',
                 gas_price: Coin = Coin(amount='0', denom=''),
                 prefix: str = DEFAULT_BECH32_PREFIX):
        self._prefix = prefix
        self.chain_id = chain_id
        if gas_price.amount != '0' and gas_price.denom == '':
            raise Exception('gas price denom can not be empty')
        self.gas_price = gas_price
        self._private_key = private_key
        self._memo = ''

    @classmethod
    def from_mnemonic(cls, mnemonic: str, path: str = ETH_DERIVATION_PATH,
                      chain_id: str = '',
                      gas_price: Coin = Coin(amount='0', denom=''),
                      prefix: str = DEFAULT_BECH32_PREFIX,
                      key_type: str = ETHSECP256K1_KEY_TYPE):
        private_key = mnemonic_to_privkey(mnemonic=mnemonic, path=path, key_type=key_type)
        return cls(private_key=private_key, chain_id=chain_id, gas_price=gas_price, prefix=prefix)

    @classmethod
    def from_mnemonic_with_index(cls, mnemonic: str, index: int = 0,
                                 chain_id: str = '',
                                 gas_price: Coin = Coin(amount='0', denom=''),
                                 prefix: str = DEFAULT_BECH32_PREFIX,
                                 key_type: str = ETHSECP256K1_KEY_TYPE):
        if key_type == ETHSECP256K1_KEY_TYPE:
            path = "m/44'/60'/0'/0/{}".format(index)
        else:
            path = "m/44'/118'/0'/0/{}".format(index)
        private_key = mnemonic_to_privkey(mnemonic=mnemonic, path=path, key_type=key_type)
        return cls(private_key=private_key, chain_id=chain_id, gas_price=gas_price, prefix=prefix)

    def with_memo(self, memo: str):
        self._memo = memo

    def has_gas_price(self) -> bool:
        return self.gas_price.denom != ''

    def with_gas_price(self, gas_prices: [Coin]):
        if len(gas_prices) == 0:
            raise Exception("gas prices is empty")
        if self.gas_price.denom == '':
            self.gas_price = gas_prices[0]
            return
        for item in gas_prices:
            if item.denom == self.gas_price.denom:
                self.gas_price = item
        if self.has_gas_price() is False:
            raise Exception("invalid gas price")

    def from_address(self, prefix: str = None) -> Address:
        prefix = self._prefix if prefix is None else prefix
        return self._private_key.to_address(prefix=prefix)

    def sign(self, msgs: [Any], fee: Fee, account_number: int, sequence: int, timeout_height: int = 0) -> Tx:
        tx_body = TxBody(messages=msgs, memo=self._memo, timeout_height=timeout_height)
        tx_body_bytes = tx_body.SerializeToString()

        single = ModeInfo.Single(mode=SIGN_MODE_DIRECT)
        mode_info = ModeInfo(single=single)
        pub_key_any = self._private_key.to_public_key().to_secp256k1_any()

        signer_info = SignerInfo(
            public_key=pub_key_any, mode_info=mode_info, sequence=sequence)
        auth_info = AuthInfo(signer_infos=[signer_info], fee=fee)
        auth_info_bytes = auth_info.SerializeToString()

        sign_doc = SignDoc(body_bytes=tx_body_bytes,
                           auth_info_bytes=auth_info_bytes,
                           chain_id=self.chain_id,
                           account_number=account_number)
        sign_doc_bytes = sign_doc.SerializeToString()
        signature = self._private_key.sign(sign_doc_bytes)
        return Tx(body=tx_body, auth_info=auth_info, signatures=[signature])



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ethermint/feemarket/v1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.ethermint.feemarket.v1 import feemarket_pb2 as ethermint_dot_feemarket_dot_v1_dot_feemarket__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$ethermint/feemarket/v1/genesis.proto\x12\x16\x65thermint.feemarket.v1\x1a&ethermint/feemarket/v1/feemarket.proto\x1a\x14gogoproto/gogo.proto\"g\n\x0cGenesisState\x12\x34\n\x06params\x18\x01 \x01(\x0b\x32\x1e.ethermint.feemarket.v1.ParamsB\x04\xc8\xde\x1f\x00\x12\x11\n\tblock_gas\x18\x03 \x01(\x04J\x04\x08\x02\x10\x03R\x08\x62\x61se_feeB.Z,github.com/evmos/ethermint/x/feemarket/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ethermint.feemarket.v1.genesis_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/evmos/ethermint/x/feemarket/types'
  _GENESISSTATE.fields_by_name['params']._options = None
  _GENESISSTATE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _globals['_GENESISSTATE']._serialized_start=126
  _globals['_GENESISSTATE']._serialized_end=229
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ethermint/feemarket/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.ethermint.feemarket.v1 import feemarket_pb2 as ethermint_dot_feemarket_dot_v1_dot_feemarket__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"ethermint/feemarket/v1/query.proto\x12\x16\x65thermint.feemarket.v1\x1a\x14gogoproto/gogo.proto\x1a&ethermint/feemarket/v1/feemarket.proto\x1a\x1cgoogle/api/annotations.proto\"\x14\n\x12QueryParamsRequest\"K\n\x13QueryParamsResponse\x12\x34\n\x06params\x18\x01 \x01(\x0b\x32\x1e.ethermint.feemarket.v1.ParamsB\x04\xc8\xde\x1f\x00\"\x15\n\x13QueryBaseFeeRequest\"T\n\x14QueryBaseFeeResponse\x12<\n\x08\x62\x61se_fee\x18\x01 \x01(\tB*\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\"\x16\n\x14QueryBlockGasRequest\"$\n\x15QueryBlockGasResponse\x12\x0b\n\x03gas\x18\x01 \x01(\x03\x32\xb9\x03\n\x05Query\x12\x89\x01\n\x06Params\x12*.ethermint.feemarket.v1.QueryParamsRequest\x1a+.ethermint.feemarket.v1.QueryParamsResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/ethermint/feemarket/v1/params\x12\x8e\x01\n\x07\x42\x61seFee\x12+.ethermint.feemarket.v1.QueryBaseFeeRequest\x1a,.ethermint.feemarket.v1.QueryBaseFeeResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /ethermint/feemarket/v1/base_fee\x12\x92\x01\n\x08\x42lockGas\x12,.ethermint.feemarket.v1.QueryBlockGasRequest\x1a-.ethermint.feemarket.v1.QueryBlockGasResponse\")\x82\xd3\xe4\x93\x02#\x12!/ethermint/feemarket/v1/block_gasB.Z,github.com/evmos/ethermint/x/feemarket/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ethermint.feemarket.v1.query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/evmos/ethermint/x/feemarket/types'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYBASEFEERESPONSE.fields_by_name['base_fee']._options = None
  _QUERYBASEFEERESPONSE.fields_by_name['base_fee']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Int'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002 \022\036/ethermint/feemarket/v1/params'
  _QUERY.methods_by_name['BaseFee']._options = None
  _QUERY.methods_by_name['BaseFee']._serialized_options = b'\202\323\344\223\002\"\022 /ethermint/feemarket/v1/base_fee'
  _QUERY.methods_by_name['BlockGas']._options = None
  _QUERY.methods_by_name['BlockGas']._serialized_options = b'\202\323\344\223\002#\022!/ethermint/feemarket/v1/block_gas'
  _globals['_QUERYPARAMSREQUEST']._serialized_start=154
  _globals['_QUERYPARAMSREQUEST']._serialized_end=174
  _globals['_QUERYPARAMSRESPONSE']._serialized_start=176
  _globals['_QUERYPARAMSRESPONSE']._serialized_end=251
  _globals['_QUERYBASEFEEREQUEST']._serialized_start=253
  _globals['_QUERYBASEFEEREQUEST']._serialized_end=274
  _globals['_QUERYBASEFEERESPONSE']._serialized_start=276
  _globals['_QUERYBASEFEERESPONSE']._serialized_end=360
  _globals['_QUERYBLOCKGASREQUEST']._serialized_start=362
  _globals['_QUERYBLOCKGASREQUEST']._serialized_end=384
  _globals['_QUERYBLOCKGASRESPONSE']._serialized_start=386
  _globals['_QUERYBLOCKGASRESPONSE']._serialized_end=422
  _globals['_QUERY']._serialized_start=425
  _globals['_QUERY']._serialized_end=866
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/group/v1/types.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x63osmos/group/v1/types.proto\x12\x0f\x63osmos.group.v1\x1a\x14gogoproto/gogo.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x19google/protobuf/any.proto\"\x8d\x01\n\x06Member\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x0e\n\x06weight\x18\x02 \x01(\t\x12\x10\n\x08metadata\x18\x03 \x01(\t\x12\x36\n\x08\x61\x64\x64\x65\x64_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01\"\\\n\rMemberRequest\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x0e\n\x06weight\x18\x02 \x01(\t\x12\x10\n\x08metadata\x18\x03 \x01(\t\"y\n\x17ThresholdDecisionPolicy\x12\x11\n\tthreshold\x18\x01 \x01(\t\x12\x37\n\x07windows\x18\x02 \x01(\x0b\x32&.cosmos.group.v1.DecisionPolicyWindows:\x12\xca\xb4-\x0e\x44\x65\x63isionPolicy\"{\n\x18PercentageDecisionPolicy\x12\x12\n\npercentage\x18\x01 \x01(\t\x12\x37\n\x07windows\x18\x02 \x01(\x0b\x32&.cosmos.group.v1.DecisionPolicyWindows:\x12\xca\xb4-\x0e\x44\x65\x63isionPolicy\"\x96\x01\n\x15\x44\x65\x63isionPolicyWindows\x12:\n\rvoting_period\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xc8\xde\x1f\x00\x98\xdf\x1f\x01\x12\x41\n\x14min_execution_period\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xc8\xde\x1f\x00\x98\xdf\x1f\x01\"\xb3\x01\n\tGroupInfo\x12\n\n\x02id\x18\x01 \x01(\x04\x12\'\n\x05\x61\x64min\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08metadata\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\x04\x12\x14\n\x0ctotal_weight\x18\x05 \x01(\t\x12\x38\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01\"H\n\x0bGroupMember\x12\x10\n\x08group_id\x18\x01 \x01(\x04\x12\'\n\x06member\x18\x02 \x01(\x0b\x32\x17.cosmos.group.v1.Member\"\xb1\x02\n\x0fGroupPolicyInfo\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08group_id\x18\x02 \x01(\x04\x12\'\n\x05\x61\x64min\x18\x03 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08metadata\x18\x04 \x01(\t\x12\x0f\n\x07version\x18\x05 \x01(\x04\x12Q\n\x0f\x64\x65\x63ision_policy\x18\x06 \x01(\x0b\x32\x14.google.protobuf.AnyB\"\xca\xb4-\x1e\x63osmos.group.v1.DecisionPolicy\x12\x38\n\ncreated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x01\"\x9f\x04\n\x08Proposal\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x36\n\x14group_policy_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08metadata\x18\x03 \x01(\t\x12+\n\tproposers\x18\x04 \x03(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x39\n\x0bsubmit_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01\x12\x15\n\rgroup_version\x18\x06 \x01(\x04\x12\x1c\n\x14group_policy_version\x18\x07 \x01(\x04\x12/\n\x06status\x18\x08 \x01(\x0e\x32\x1f.cosmos.group.v1.ProposalStatus\x12>\n\x12\x66inal_tally_result\x18\t \x01(\x0b\x32\x1c.cosmos.group.v1.TallyResultB\x04\xc8\xde\x1f\x00\x12?\n\x11voting_period_end\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01\x12@\n\x0f\x65xecutor_result\x18\x0b \x01(\x0e\x32\'.cosmos.group.v1.ProposalExecutorResult\x12&\n\x08messages\x18\x0c \x03(\x0b\x32\x14.google.protobuf.Any:\x04\x88\xa0\x1f\x00\"k\n\x0bTallyResult\x12\x11\n\tyes_count\x18\x01 \x01(\t\x12\x15\n\rabstain_count\x18\x02 \x01(\t\x12\x10\n\x08no_count\x18\x03 \x01(\t\x12\x1a\n\x12no_with_veto_count\x18\x04 \x01(\t:\x04\x88\xa0\x1f\x00\"\xbe\x01\n\x04Vote\x12\x13\n\x0bproposal_id\x18\x01 \x01(\x04\x12\'\n\x05voter\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12+\n\x06option\x18\x03 \x01(\x0e\x32\x1b.cosmos.group.v1.VoteOption\x12\x10\n\x08metadata\x18\x04 \x01(\t\x12\x39\n\x0bsubmit_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xc8\xde\x1f\x00\x90\xdf\x1f\x01*\x8f\x01\n\nVoteOption\x12\x1b\n\x17VOTE_OPTION_UNSPECIFIED\x10\x00\x12\x13\n\x0fVOTE_OPTION_YES\x10\x01\x12\x17\n\x13VOTE_OPTION_ABSTAIN\x10\x02\x12\x12\n\x0eVOTE_OPTION_NO\x10\x03\x12\x1c\n\x18VOTE_OPTION_NO_WITH_VETO\x10\x04\x1a\x04\x88\xa3\x1e\x00*\xce\x01\n\x0eProposalStatus\x12\x1f\n\x1bPROPOSAL_STATUS_UNSPECIFIED\x10\x00\x12\x1d\n\x19PROPOSAL_STATUS_SUBMITTED\x10\x01\x12\x1c\n\x18PROPOSAL_STATUS_ACCEPTED\x10\x02\x12\x1c\n\x18PROPOSAL_STATUS_REJECTED\x10\x03\x12\x1b\n\x17PROPOSAL_STATUS_ABORTED\x10\x04\x12\x1d\n\x19PROPOSAL_STATUS_WITHDRAWN\x10\x05\x1a\x04\x88\xa3\x1e\x00*\xba\x01\n\x16ProposalExecutorResult\x12(\n$PROPOSAL_EXECUTOR_RESULT_UNSPECIFIED\x10\x00\x12$\n PROPOSAL_EXECUTOR_RESULT_NOT_RUN\x10\x01\x12$\n PROPOSAL_EXECUTOR_RESULT_SUCCESS\x10\x02\x12$\n PROPOSAL_EXECUTOR_RESULT_FAILURE\x10\x03\x1a\x04\x88\xa3\x1e\x00\x42&Z$github.com/cosmos/cosmos-sdk/x/groupb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.group.v1.types_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z$github.com/cosmos/cosmos-sdk/x/group'
  _VOTEOPTION._options = None
  _VOTEOPTION._serialized_options = b'\210\243\036\000'
  _PROPOSALSTATUS._options = None
  _PROPOSALSTATUS._serialized_options = b'\210\243\036\000'
  _PROPOSALEXECUTORRESULT._options = None
  _PROPOSALEXECUTORRESULT._serialized_options = b'\210\243\036\000'
  _MEMBER.fields_by_name['address']._options = None
  _MEMBER.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MEMBER.fields_by_name['added_at']._options = None
  _MEMBER.fields_by_name['added_at']._serialized_options = b'\310\336\037\000\220\337\037\001'
  _MEMBERREQUEST.fields_by_name['address']._options = None
  _MEMBERREQUEST.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _THRESHOLDDECISIONPOLICY._options = None
  _THRESHOLDDECISIONPOLICY._serialized_options = b'\312\264-\016DecisionPolicy'
  _PERCENTAGEDECISIONPOLICY._options = None
  _PERCENTAGEDECISIONPOLICY._serialized_options = b'\312\264-\016DecisionPolicy'
  _DECISIONPOLICYWINDOWS.fields_by_name['voting_period']._options = None
  _DECISIONPOLICYWINDOWS.fields_by_name['voting_period']._serialized_options = b'\310\336\037\000\230\337\037\001'
  _DECISIONPOLICYWINDOWS.fields_by_name['min_execution_period']._options = None
  _DECISIONPOLICYWINDOWS.fields_by_name['min_execution_period']._serialized_options = b'\310\336\037\000\230\337\037\001'
  _GROUPINFO.fields_by_name['admin']._options = None
  _GROUPINFO.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _GROUPINFO.fields_by_name['created_at']._options = None
  _GROUPINFO.fields_by_name['created_at']._serialized_options = b'\310\336\037\000\220\337\037\001'
  _GROUPPOLICYINFO.fields_by_name['address']._options = None
  _GROUPPOLICYINFO.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _GROUPPOLICYINFO.fields_by_name['admin']._options = None
  _GROUPPOLICYINFO.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _GROUPPOLICYINFO.fields_by_name['decision_policy']._options = None
  _GROUPPOLICYINFO.fields_by_name['decision_policy']._serialized_options = b'\312\264-\036cosmos.group.v1.DecisionPolicy'
  _GROUPPOLICYINFO.fields_by_name['created_at']._options = None
  _GROUPPOLICYINFO.fields_by_name['created_at']._serialized_options = b'\310\336\037\000\220\337\037\001'
  _GROUPPOLICYINFO._options = None
  _GROUPPOLICYINFO._serialized_options = b'\210\240\037\000\350\240\037\001'
  _PROPOSAL.fields_by_name['group_policy_address']._options = None
  _PROPOSAL.fields_by_name['group_policy_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _PROPOSAL.fields_by_name['proposers']._options = None
  _PROPOSAL.fields_by_name['proposers']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _PROPOSAL.fields_by_name['submit_time']._options = None
  _PROPOSAL.fields_by_name['submit_time']._serialized_options = b'\310\336\037\000\220\337\037\001'
  _PROPOSAL.fields_by_name['final_tally_result']._options = None
  _PROPOSAL.fields_by_name['final_tally_result']._serialized_options = b'\310\336\037\000'
  _PROPOSAL.fields_by_name['voting_period_end']._options = None
  _PROPOSAL.fields_by_name['voting_period_end']._serialized_options = b'\310\336\037\000\220\337\037\001'
  _PROPOSAL._options = None
  _PROPOSAL._serialized_options = b'\210\240\037\000'
  _TALLYRESULT._options = None
  _TALLYRESULT._serialized_options = b'\210\240\037\000'
  _VOTE.fields_by_name['voter']._options = None
  _VOTE.fields_by_name['voter']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _VOTE.fields_by_name['submit_time']._options = None
  _VOTE.fields_by_name['submit_time']._serialized_options = b'\310\336\037\000\220\337\037\001'
  _globals['_VOTEOPTION']._serialized_start=2241
  _globals['_VOTEOPTION']._serialized_end=2384
  _globals['_PROPOSALSTATUS']._serialized_start=2387
  _globals['_PROPOSALSTATUS']._serialized_end=2593
  _globals['_PROPOSALEXECUTORRESULT']._serialized_start=2596
  _globals['_PROPOSALEXECUTORRESULT']._serialized_end=2782
  _globals['_MEMBER']._serialized_start=190
  _globals['_MEMBER']._serialized_end=331
  _globals['_MEMBERREQUEST']._serialized_start=333
  _globals['_MEMBERREQUEST']._serialized_end=425
  _globals['_THRESHOLDDECISIONPOLICY']._serialized_start=427
  _globals['_THRESHOLDDECISIONPOLICY']._serialized_end=548
  _globals['_PERCENTAGEDECISIONPOLICY']._serialized_start=550
  _globals['_PERCENTAGEDECISIONPOLICY']._serialized_end=673
  _globals['_DECISIONPOLICYWINDOWS']._serialized_start=676
  _globals['_DECISIONPOLICYWINDOWS']._serialized_end=826
  _globals['_GROUPINFO']._serialized_start=829
  _globals['_GROUPINFO']._serialized_end=1008
  _globals['_GROUPMEMBER']._serialized_start=1010
  _globals['_GROUPMEMBER']._serialized_end=1082
  _globals['_GROUPPOLICYINFO']._serialized_start=1085
  _globals['_GROUPPOLICYINFO']._serialized_end=1390
  _globals['_PROPOSAL']._serialized_start=1393
  _globals['_PROPOSAL']._serialized_end=1936
  _globals['_TALLYRESULT']._serialized_start=1938
  _globals['_TALLYRESULT']._serialized_end=2045
  _globals['_VOTE']._serialized_start=2048
  _globals['_VOTE']._serialized_end=2238
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from fxsdk.x.fx.legacy.other import query_pb2 as fx_dot_legacy_dot_other_dot_query__pb2


class QueryStub(object):
    """Deprecated: Query
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.FxGasPrice = channel.unary_unary(
                '/fx.other.Query/FxGasPrice',
                request_serializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceRequest.SerializeToString,
                response_deserializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceResponse.FromString,
                )
        self.GasPrice = channel.unary_unary(
                '/fx.other.Query/GasPrice',
                request_serializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceRequest.SerializeToString,
                response_deserializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceResponse.FromString,
                )


class QueryServicer(object):
    """Deprecated: Query
    """

    def FxGasPrice(self, request, context):
        """Deprecated: please use cosmos.base.node.v1beta1.Service.Config
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GasPrice(self, request, context):
        """Deprecated: please use cosmos.base.node.v1beta1.Service.Config
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'FxGasPrice': grpc.unary_unary_rpc_method_handler(
                    servicer.FxGasPrice,
                    request_deserializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceRequest.FromString,
                    response_serializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceResponse.SerializeToString,
            ),
            'GasPrice': grpc.unary_unary_rpc_method_handler(
                    servicer.GasPrice,
                    request_deserializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceRequest.FromString,
                    response_serializer=fx_dot_legacy_dot_other_dot_query__pb2.GasPriceResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fx.other.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Deprecated: Query
    """

    @staticmethod
    def FxGasPrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.other.Query/FxGasPrice',
            fx_dot_legacy_dot_other_dot_query__pb2.GasPriceRequest.SerializeToString,
            fx_dot_legacy_dot_other_dot_query__pb2.GasPriceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GasPrice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.other.Query/GasPrice',
            fx_dot_legacy_dot_other_dot_query__pb2.GasPriceRequest.SerializeToString,
            fx_dot_legacy_dot_other_dot_query__pb2.GasPriceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import unittest

from fxsdk.wallet.address import Address
from fxsdk.wallet.key import PublicKey, PrivateKey, SECP256K1_KEY_TYPE
from fxsdk.wallet.key import mnemonic_to_privkey
from eth_hash.auto import keccak as keccak_256


class TestWallet(unittest.TestCase):
    def test_seed_to_address(self):
        private_key = mnemonic_to_privkey('test test test test test test test test test test test junk')
        self.assertEqual(private_key.to_hex(), 'ac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80')
        self.assertEqual(private_key.to_public_key().to_hex(),
                         '038318535b54105d4a7aae60c08fc45f9687181b4fdfc625bd1a753fa7397fed75')
        self.assertEqual(private_key.to_address(prefix='mx').to_string(),
                         'mx17w0adeg64ky0daxwd2ugyuneellmjgnxqezt98')

    def test_public_key(self):
        pub_key_bytes = bytes.fromhex('038318535b54105d4a7aae60c08fc45f9687181b4fdfc625bd1a753fa7397fed75')
        pub_key = PublicKey(pub_key=pub_key_bytes, key_type=SECP256K1_KEY_TYPE)
        self.assertEqual(pub_key.to_address().to_string(),
                         'fx15428vq2uzwhm3taey9sr9x5vm6tk78ewryy2rg')

    def test_public_key_protobuf_serialization(self):
        pub_key_bytes = bytes.fromhex('02ad0593f884c5784b69d55b8f2ff244443309abd31c90bad2e440b5171d136098')
        pub_key = PublicKey(pub_key=pub_key_bytes, key_type=SECP256K1_KEY_TYPE)
        self.assertEqual(pub_key.to_secp256k1_any().SerializeToString().hex(),
                         '0a1f2f636f736d6f732e63727970746f2e736563703235366b312e5075624b657912230a2102ad0593f884c5784b69d55b8f2ff244443309abd31c90bad2e440b5171d136098')

    def test_private_key_sign(self):
        priv_key_bytes = bytes.fromhex('ac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80')
        priv_key = PrivateKey(priv_key_bytes)
        self.assertEqual(priv_key.to_address('mx').to_string(), 'mx17w0adeg64ky0daxwd2ugyuneellmjgnxqezt98')
        self.assertEqual(priv_key.sign(priv_key_bytes).hex(),
                         '93d0d025c08961da1fc345352a89ecaa60d95715238c1cb699a2cedb3d2db6ed2b8d76d077dc7e1053d7192bf39c2a8ba61bd6e6c6d98d129c2405b21adf2d1800')

    def test_private_key_sign2(self):
        priv_key_bytes = bytes.fromhex('e64e7928d4f6c06f01fefd31f760c51f59a16426e792761cd00529b76501c8a0')
        priv_key = PrivateKey(priv_key_bytes, SECP256K1_KEY_TYPE)
        self.assertEqual(priv_key.to_address('mx').to_string(), 'mx15yk64u7zc9g9k2yr2wmzeva5qgwxps6y9kkkhe')
        self.assertEqual(priv_key.sign(priv_key_bytes).hex(),
                         'ad489e841a6b2806d80c11daa3e1382bbba7e00bf342f88d31df6154c89c3b6b01bdd3055b9e4e94fbe7558b57e6b578eed881f064a47867617307b0bde1ddba')

    def test_keccak256(self):
        data = keccak_256(bytes.fromhex('ac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80'))
        self.assertEqual(data.hex(), '5b2ef0b493f7d7368b311b19a1165c7ad9315c5bec1256fdcf98de11b940aed2')

    def test_address(self):
        address = Address.from_str('mx17w0adeg64ky0daxwd2ugyuneellmjgnxqezt98')
        self.assertEqual(address.to_string(), 'mx17w0adeg64ky0daxwd2ugyuneellmjgnxqezt98')
        self.assertEqual(address.to_hex(),'0xf39Fd6e51aad88F6F4ce6aB8827279cffFb92266')


if __name__ == '__main__':
    unittest.main()
